@@ -39,7 +39,7 @@ CONF.import_group('keystone_authtoken', 'keystonemiddleware.auth_token')
 
 LOG = logging.getLogger(__name__)
 
-NOVA_API_VERSION = "2.14"
+NOVA_API_VERSION = "2.53"
 
 nova_extensions = [ext for ext in
                    nova_client.discover_extensions(NOVA_API_VERSION)
@@ -149,19 +149,19 @@ class API(object):
     @translate_nova_exception
     def enable_disable_service(self, context, host_name, enable=False,
                                reason=None):
-        """Enable or disable the service specified by hostname and binary."""
+        """Enable or disable the service specified by nova service id."""
         nova = novaclient(context)
+        service = nova.services.list(host=host_name, binary='nova-compute')[0]
 
         if not enable:
             LOG.info('Disable nova-compute on %s', host_name)
             if reason:
-                nova.services.disable_log_reason(host_name, 'nova-compute',
-                                                 reason)
+                nova.services.disable_log_reason(service.id, reason)
             else:
-                nova.services.disable(host_name, 'nova-compute')
+                nova.services.disable(service.id)
         else:
             LOG.info('Enable nova-compute on %s', host_name)
-            nova.services.enable(host_name, 'nova-compute')
+            nova.services.enable(service.id)
 
     @translate_nova_exception
     def is_service_down(self, context, host_name, binary):
@@ -243,3 +243,20 @@ class API(object):
         msg = ('Call unlock server command for instance %(uuid)s')
         LOG.info(msg, {'uuid': uuid})
         return nova.servers.unlock(uuid)
+
+    @translate_nova_exception
+    def hypervisor_search(self, context, hypervisor_name):
+        """Search hypervisor with case sensitive hostname."""
+        nova = novaclient(context)
+        msg = ("Call hypervisor search command to get list of matching "
+               "hypervisor name '%(hypervisor_name)s'")
+        LOG.info(msg, {'hypervisor_name': hypervisor_name})
+        try:
+            hypervisors_list = nova.hypervisors.search(hypervisor_name)
+            if hypervisor_name not in [host.hypervisor_hostname for host in
+                            hypervisors_list]:
+                raise exception.HypervisorNotFoundByName(
+                    hypervisor_name=hypervisor_name)
+        except nova_exception.NotFound:
+            raise exception.HypervisorNotFoundByName(
+                hypervisor_name=hypervisor_name)
